//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#pragma once
#include "SelNAFCDlg.h"
#include <Iphlpapi.h>

class CNAFC 
{
public:
   CNAFC();
   ~CNAFC();

   // Update the history list. Must be called every cycle
   void Process();

   // Check if the NAFC is available on this computer
   bool IsNAFCAvailable() const {return (m_fGetIfEntry != NULL);}

   // Full NAFC bandwidth control   
   float GetMaxDownload() const {return m_maxDownloadLimit;}
   float GetMaxUpload() const	{return m_maxUploadLimit;}

   //Xman new adapter selection
	bool	CheckAdapterIndex(const uint32& highid, const bool& checkonlyexisting = false);
   void SetWasNAFCLastActive(bool in) {wasNAFCLastActive=in;}	
   bool GetwasNAFCLastActive() {return wasNAFCLastActive;} 

private:

   // [ionix] - Ratio
   float   GetMaxDownloadOnUpload(float upload, float maxdownload, bool isConnectionOH);

   // Adapter access
	//DWORD GetAdapterIndex();
   DWORD m_currentAdapterIndex;

	//Xman new adapter selection
   bool wasNAFCLastActive;

   // Type definition
   #pragma pack(1)
   struct Statistic {
      Statistic() {}
      Statistic(const Statistic& ref, uint32 time)
      :   eMuleOutOverall(ref.eMuleOutOverall),
		  eMuleInOverall(ref.eMuleInOverall),
		  networkOut(ref.networkOut),
          networkIn(ref.networkIn),
          timeStamp(time) {}

		uint64 eMuleOutOverall; // Data+Control
		uint64 eMuleInOverall; // Data+Control
		uint64 networkOut; // DataFlow of the network Adapter
		uint64 networkIn;
		uint32 timeStamp; // Use a time stamp to compensate the inaccuracy of the timer (based on enkeyDEV(Ottavio84))
   };
   #pragma pack()
   typedef CList<Statistic> StatisticHistory; // Use MS container for its memory management

   StatisticHistory m_statisticHistory; // History for the graphic
   Statistic m_statistic; // Current value

   float m_maxDownloadLimit; // Used for auto U/D limits
   float m_maxUploadLimit;

   //Xman thread save
   mutable CCriticalSection m_statisticLocker;  

	//detect adapter changes
	void	SelectAdapter(const DWORD& adapter);

    // Keep last result to detect an overflow
	//DWORD m_networkOut;
	//DWORD m_networkIn;

   bool m_errorTraced;

   // Dynamic access to the iphlpapi.dll
   typedef DWORD (WINAPI *GETNUMBEROFINTERFACES)(PDWORD pdwNumIf);
   typedef DWORD (WINAPI *GETIFTABLE)(PMIB_IFTABLE pIfTable, PULONG pdwSize, BOOL bOrder);
   typedef DWORD (WINAPI *GETIPADDRTABLE)(PMIB_IPADDRTABLE pIpAddrTable, PULONG pdwSize, BOOL bOrder);
   typedef DWORD (WINAPI *GETIFENTRY)(PMIB_IFROW pIfRow);

   HINSTANCE m_hIphlpapi;
   GETIFTABLE m_fGetIfTable;
   GETIPADDRTABLE m_fGetIpAddrTable;
   GETIFENTRY m_fGetIfEntry;
   GETNUMBEROFINTERFACES m_fGetNumberOfInterfaces;

private:
   // Don't allow canonical behavior
   CNAFC(const CNAFC&);
   CNAFC& operator=(const CNAFC&);

//>>> NAFC Selection
private:
   struct NAFCInfo
   {
	   CString description;
	   DWORD type;
	   DWORD index;
	   DWORD mtu;
	   DWORD ip;
   };

public:
	NAFCInfo m_NAFCInfo[8];
	DWORD	NafcNumber;
	DWORD	GetSelectedIndex() const { return m_currentAdapterIndex; }
	void	SelectNAFC();
//<<< NAFC Selection
};