//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "stdafx.h"
#include <math.h>
#include "emule.h"
#include "UploadBandwidthThrottler.h"
#include "opcodes.h"
#include "LastCommonRouteFinder.h"
#include "OtherFunctions.h"
#include "emuledlg.h"
// Pawcio: BC
#include "preferences.h"
#include "updownclient.h"
#include "listensocket.h"
// <--- Pawcio: BC

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

/**
* The constructor starts the thread.
*/
UploadBandwidthThrottler::UploadBandwidthThrottler(void) {
	m_SentBytesSinceLastCall = 0;

	threadEndedEvent = new CEvent(0, 1);
	pauseEvent = new CEvent(TRUE, TRUE);

	doRun = true;
	timeUp = thePrefs.GetBCTimeUp();
	AfxBeginThread(RunProc, (LPVOID)this);
}

/**
* The destructor stops the thread. If the thread has already stoppped, destructor does nothing.
*/
UploadBandwidthThrottler::~UploadBandwidthThrottler(void) {
	EndThread();
	delete threadEndedEvent;
	delete pauseEvent;
}

// Pawcio: BC - removed

/**
* Find out how many bytes that has been put on the sockets since the last call to this
* method. Includes overhead of control packets.
*
* @return the number of bytes that has been put on the sockets since the last call
*/
uploadValues UploadBandwidthThrottler::GetNumberOfSentBytesSinceLastCallAndReset() {
	dataLocker.Lock();
	uploadValues ret = {m_SentBytesSinceLastCall, bandwidthUsed, uploadinglist.GetCount()};
	m_SentBytesSinceLastCall = 0;
	dataLocker.Unlock();

	return ret;
}

/**
* @param socket the address to the socket that should be added to the list. If the address is NULL,
*               this method will do nothing.
*/
// Pawcio: BC
void UploadBandwidthThrottler::AddToStandardList(CClientReqSocket* socket, bool toReserved) {
	sendLocker.Lock();
	
	AddToStandardListNoLock(socket, toReserved);

	sendLocker.Unlock();
}

void UploadBandwidthThrottler::AddToStandardListNoLock(CClientReqSocket* socket, bool toReserved) {
	if (socket && !socket->onUpQueue){
		socket->ResetProcessRate();
		socket->onUpQueue = true;
		if (toReserved){
			ASSERT(!reserveduploadinglist.Find(socket));
			socket->onReserved = true;
			reserveduploadinglist.AddTail(socket);
			prioritylist.AddTail(socket);

		}
		else {
			ASSERT(!uploadinglist.Find(socket));
			socket->onReserved = false;
			uploadinglist.AddTail(socket);
			prioritylist.AddTail(socket);
		}
	}
}

void UploadBandwidthThrottler::MoveFromReserved(){
	sendLocker.Lock();
	if (!reserveduploadinglist.IsEmpty()){
		if (reservedFeedingPos == reserveduploadinglist.GetHeadPosition())
			reservedFeedingPos = NULL;
		CClientReqSocket* moving = reserveduploadinglist.RemoveHead();
		moving->onReserved = false;
		moving->SetLastBlockSent();
		uploadinglist.AddTail(moving);
	}
	sendLocker.Unlock();
}

// Pawcio: BC
bool UploadBandwidthThrottler::AddToBlockedQueue(ThrottledControlSocket* socket){
	if (socket){
		sendLocker.Lock();
		bool ret = socket->onBlockedQueue;
		if (socket && !socket->onBlockedQueue) {
			socket->onBlockedQueue = true;
			blockedSockets.AddTail(socket);
			ret = socket->onBlockedQueue;
		}
		sendLocker.Unlock();
		return ret;
	}
	return false;
}

/**
* Remove a socket from the list of sockets that have upload slots.
*
* @param socket the address of the socket that should be removed from the list. If this socket
*               does not exist in the list, this method will do nothing.
*/
bool UploadBandwidthThrottler::RemoveFromStandardList(CClientReqSocket* socket) {
	bool returnValue = false;
	if (socket){
		sendLocker.Lock();

		returnValue = RemoveFromStandardListNoLock(socket);

		sendLocker.Unlock();
	}
	return returnValue;
}

/**
* Remove a socket from the list of sockets that have upload slots. NOT THREADSAFE!
* This is an internal method that doesn't take the necessary lock before it removes
* the socket. This method should only be called when the current thread already owns
* the sendLocker lock!
*
* @param socket address of the socket that should be removed from the list. If this socket
*               does not exist in the list, this method will do nothing.
*/
bool UploadBandwidthThrottler::RemoveFromStandardListNoLock(CClientReqSocket* socket) {
	bool found = false;
	if (socket && socket->onUpQueue){
		POSITION pos = prioritylist.Find(socket);
		if (pos)
			prioritylist.RemoveAt(pos);
		else if (socket != prioritySocket)
			ASSERT(false);
		if (socket->onReserved) {
			pos = reserveduploadinglist.Find(socket);
			if (pos){
				found = true;
				socket->onReserved = false;
				socket->onUpQueue = false;
				if (reservedFeedingPos == pos){
					POSITION next = pos;
					reserveduploadinglist.GetNext(next);
					reservedFeedingPos = next;
				}
				reserveduploadinglist.RemoveAt(pos);
			}
			else
				ASSERT(false);
		}
		else {
			// Find the slot
			pos = uploadinglist.Find(socket);
			if (pos){
				found = true;
				socket->onReserved = false;
				socket->onUpQueue = false;
				if (socket == prioritySocket) {
					socket->isForPriorityClient = false;
					prioritySocket=NULL;
				}
				if (feedingPos == pos){
					POSITION next = pos;
					uploadinglist.GetNext(next);
					feedingPos = next;
				}
				uploadinglist.RemoveAt(pos);
			}
			else
				ASSERT(false);
		}
	}
	return found;
}

/**
* Notifies the send thread that it should try to call controlpacket send
* for the supplied socket. It is allowed to call this method several times
* for the same socket, without having controlpacket send called for the socket
* first.
*
* @param socket address to the socket that requests to have controlpacket send
*               to be called on it
*/
bool UploadBandwidthThrottler::QueueForSendingControlPacket(ThrottledControlSocket* socket) {
	if (socket){
		// Get critical section
		sendLocker.Lock();
		bool ret = socket->onUpControlQueue;
		if(doRun && socket && !socket->onUpControlQueue) {
			socket->onUpControlQueue = true;
			m_ControlQueue_list.AddTail(socket);
			ret = socket->onUpControlQueue;
		}

		// End critical section
		sendLocker.Unlock();
		return ret;
	}
	return false;
}

/**
* Remove the socket from all lists and queues. This will make it safe to
* erase/delete the socket. It will also cause the main thread to stop calling
* send() for the socket.
*
* @param socket address to the socket that should be removed
*/
void UploadBandwidthThrottler::RemoveFromAllQueues(ThrottledControlSocket* socket, bool lock) {
	if(lock) {
		// Get critical section
		sendLocker.Lock();
	}

	if(doRun) {
		// Remove this socket from control packet queue
		POSITION pos = m_ControlQueue_list.Find(socket);
		if (pos){
			socket->onUpControlQueue = false;
			m_ControlQueue_list.RemoveAt(pos);
		}
		pos = blockedSockets.Find(socket);
		if (pos){
			socket->onBlockedQueue = false;
			blockedSockets.RemoveAt(pos);
		}
	}
	if(lock) {
		// End critical section
		sendLocker.Unlock();
	}
}

void UploadBandwidthThrottler::RemoveFromAllQueues(ThrottledFileSocket* socket) {
	if (socket){
		// Get critical section
		sendLocker.Lock();

		if(doRun) {
			RemoveFromAllQueues(socket, false);
		}

		// End critical section
		sendLocker.Unlock();
	}
}

/**
* Make the thread exit. This method will not return until the thread has stopped
* looping. This guarantees that the thread will not access the CEMSockets after this
* call has exited.
*/
void UploadBandwidthThrottler::EndThread() {
	sendLocker.Lock();

	// signal the thread to stop looping and exit.
	doRun = false;

	sendLocker.Unlock();

	Pause(false);

	// wait for the thread to signal that it has stopped looping.
	threadEndedEvent->Lock();
}

void UploadBandwidthThrottler::Pause(bool paused) {
	if(paused) {
		pauseEvent->ResetEvent();
	} else {
		pauseEvent->SetEvent();
	}
}

/**
* Start the thread. Called from the constructor in this class.
*
* @param pParam
*
* @return
*/
UINT AFX_CDECL UploadBandwidthThrottler::RunProc(LPVOID pParam) {
	DbgSetThreadName("UploadBandwidthThrottler");
	InitThreadLocale();
	UploadBandwidthThrottler* uploadBandwidthThrottler = (UploadBandwidthThrottler*)pParam;

	return uploadBandwidthThrottler->RunInternal();
}

/**
* @return always returns 0.
*/
// Pawcio: BC - changed
UINT UploadBandwidthThrottler::RunInternal() {
	DWORD lastLoopTick = ::GetTickCount();
	float allowedDataRate = 0;

	prioritySocket=0;
	feedingPos=0;
	reservedFeedingPos=0;
	toSend=0;
	SentToNormal=0;
	SentToReserved=0;
	nextMinPacketSize=UINT_MAX;
	lastReOrder=lastLoopTick;
	bandwidthUsed=false;
	SocketSentBytes sent;

	while(doRun) {
		pauseEvent->Lock();

		DWORD timeSinceLastLoop = ::GetTickCount() - lastLoopTick;

		// Get current speed from UploadSpeedSense
		allowedDataRate = theApp.BC->GetMaxUpload(); // Pawcio: BC

		uint32 minFragSize = thePrefs.GetMSS();
		if (allowedDataRate < 5.0)
			minFragSize = 536;
		else if (allowedDataRate >= 30.0)
			minFragSize *= 2;

#define TIME_BETWEEN_UPLOAD_LOOPS timeUp
		// sleep for just as long as we need to get back to having minFragSize/2 bytes to send
		uint32 sleepTime = max((minFragSize/2)/allowedDataRate, TIME_BETWEEN_UPLOAD_LOOPS);

		uint32 toSleep = TIME_BETWEEN_UPLOAD_LOOPS;
		if(timeSinceLastLoop < sleepTime)
			toSleep = sleepTime - timeSinceLastLoop;
		if (toSleep < 1)
			toSleep = 1;
		else if (toSleep > 250)
			toSleep = 250;
		Sleep(toSleep);

		const DWORD thisLoopTick = ::GetTickCount();
		timeSinceLastLoop = thisLoopTick - lastLoopTick;
		lastLoopTick = thisLoopTick;

		if (sentToNormalList.GetCount()>0) 
		{
			while (sentToNormalList.GetTail().timestamp - sentToNormalList.GetHead().timestamp  > MAXAVERAGETIME/2)
				DataSentToNormal -= sentToNormalList.RemoveHead().datalen;
		}
		else
			DataSentToNormal = 0;

		if (sentToReservedList.GetCount()>0) 
		{
			while (sentToReservedList.GetTail().timestamp - sentToReservedList.GetHead().timestamp  > MAXAVERAGETIME/2)
				DataSentToReserved -= sentToReservedList.RemoveHead().datalen;
		}
		else
			DataSentToReserved = 0;

		uint32 datarateX = 0;

		if (allowedDataRate == UNLIMITED)
			toSend = 0x7fffffff; //max signed value;
		else {
			uint32 currLoopAddOn = allowedDataRate * 1.024f * timeSinceLastLoop;

			if (toSend > 0 && toSend + (int) currLoopAddOn < 0)
				toSend = 0;
			toSend += currLoopAddOn;
			if (toSend > allowedDataRate * 1.024f * 1000) {
				toSend = allowedDataRate * 1.024f * 1000;
			}
		}

		POSITION pos = NULL;
		POSITION prev_pos = NULL;
		ThrottledControlSocket* cur_socket = NULL;
		uint32 packetSize;

		bool cumulate = thePrefs.IsCumulateBandwidth();
		sendLocker.Lock();
		//When upload limit is too low don't allow 
		//overhead (control packets) eats the whole bandwidth
		bool prioritySend = false;
		//Serve blocked sockets
		pos = blockedSockets.GetHeadPosition();
		while (pos){
			prev_pos=pos;
			cur_socket = blockedSockets.GetNext(pos);
			if (cur_socket->IsConnected()){
				packetSize = cur_socket->GetBlockedPacketFragSize(minFragSize);
				if (toSend > (int)packetSize || cur_socket->IsPrioritySend() && cur_socket->GetControlPacketFragSize(minFragSize)){
					cur_socket->Send(packetSize);
					if (!cur_socket->IsBusy()){
						cur_socket->onBlockedQueue = false;
						blockedSockets.RemoveAt(prev_pos);
					}
				}
			}
		}
		//Don't allow user to be removed from upload queue when there is to low bandwidth
		pos = uploadinglist.GetHeadPosition();
		prev_pos = NULL;
		CClientReqSocket* cur_clientSocket = NULL;
		CClientReqSocket* prev_clientSocket = NULL;
		while (pos){
			cur_clientSocket = uploadinglist.GetAt(pos);
			cur_clientSocket->processed = 0;
			if (cumulate && allowedDataRate != UNLIMITED && cur_clientSocket->GetTimeSinceLastSent() > 20000 && cur_clientSocket != prioritySocket && uploadinglist.GetCount() > 1){
				POSITION oldPos = pos;
				CClientReqSocket* moving = uploadinglist.GetNext(pos);
				if (feedingPos == oldPos)
					feedingPos = pos;
				uploadinglist.RemoveAt(oldPos);
				moving->onReserved = true;
				reserveduploadinglist.AddHead(moving);
				continue;
			}
			if (cur_clientSocket && cur_clientSocket->isReadyToSendFileData()){
				packetSize = cur_clientSocket->GetDataPacketFragSize(minFragSize);
				if (cur_clientSocket->GetTimeSinceLastKeepAliveSent() > (uint32) minFragSize * 2){
					if (toSend > (int)packetSize){
						sent = cur_clientSocket->Send(packetSize);
						datarateX += sent.sentBytesStandardPackets;
						cur_clientSocket->processed = 1;
						cur_clientSocket->SetTimeForSend(thisLoopTick, true);
					}
					else if (cur_clientSocket->GetTimeSinceLastKeepAliveSent() > (uint32) minFragSize * 5)
						prioritySend = true;
				}
				if (thisLoopTick - lastReOrder > 1000 && prev_clientSocket && prev_clientSocket != prioritySocket && prev_clientSocket->GetAvgRatio() < cur_clientSocket->GetAvgRatio() && cumulate){
					uploadinglist.RemoveAt(pos);
					uploadinglist.InsertBefore(prev_pos, cur_clientSocket);
					pos = prev_pos;
				}
				cur_clientSocket->m_uBusyTicks = 0;
			}
			else {
				cur_clientSocket->m_uReadyTicks = 0;
				cur_clientSocket->m_uBusyTicks++;
				cur_clientSocket->CalcRatio(true);
			}
			if (pos != prev_pos){
				prev_pos = pos;
				prev_clientSocket = cur_clientSocket;
			}
			uploadinglist.GetNext(pos);
		}

		pos=reserveduploadinglist.GetHeadPosition();
		prev_pos=NULL;
		prev_clientSocket=NULL;
		while (pos){
			cur_clientSocket = reserveduploadinglist.GetAt(pos);
			cur_clientSocket->processed=0;
			if (cur_clientSocket && cur_clientSocket->isReadyToSendFileData()){
				packetSize=cur_clientSocket->GetDataPacketFragSize(minFragSize);
				if (cur_clientSocket->GetTimeSinceLastKeepAliveSent() > (uint32) minFragSize * 2){
					if (toSend > (int)packetSize){
						sent = cur_clientSocket->Send(packetSize);
						datarateX += sent.sentBytesStandardPackets;
						cur_clientSocket->processed=1;
						cur_clientSocket->SetTimeForSend(thisLoopTick, true);
					}
					else if (cur_clientSocket->GetTimeSinceLastKeepAliveSent() > (uint32) minFragSize * 5)
						prioritySend=true;
				}
				if (thisLoopTick - lastReOrder > 1000 && prev_clientSocket && prev_clientSocket->GetAvgRatio() < cur_clientSocket->GetAvgRatio() && cumulate){
					reserveduploadinglist.RemoveAt(pos);
					reserveduploadinglist.InsertBefore(prev_pos, cur_clientSocket);
					pos = prev_pos;
				}
				cur_clientSocket->m_uBusyTicks = 0;
			}
			else {
				cur_clientSocket->m_uReadyTicks = 0;
				cur_clientSocket->m_uBusyTicks++;
				cur_clientSocket->CalcRatio(true);
			}
			if (pos != prev_pos){
				prev_pos = pos;
				prev_clientSocket = cur_clientSocket;
			}
			reserveduploadinglist.GetNext(pos);
		}

		// Move socket from reserved to normal list if not enough data is sent
		if (thisLoopTick - lastReOrder > 1000 && (cumulate || thePrefs.IsMinimizeOpenedSlots())){
			if ( DataSentToReserved*10 > DataSentToNormal && DataSentToReserved > 5000 ||
			DataSentToReserved > 50000 || DataSentToReserved * 3 > DataSentToNormal ){
				DataSentToReserved = 0;
				sentToReservedList.RemoveAll();
				if (!reserveduploadinglist.IsEmpty()){
					if (reservedFeedingPos == reserveduploadinglist.GetHeadPosition())
						reservedFeedingPos = NULL;
					CClientReqSocket* moving = reserveduploadinglist.RemoveHead();
					moving->onReserved = false;
					moving->SetLastBlockSent();
					moving->ConsiderReservedTime();
					uploadinglist.AddTail(moving);
				}
			}
		}

		if (thisLoopTick - lastReOrder > 1000)
			lastReOrder = thisLoopTick;
		//Serve control packets
		pos = m_ControlQueue_list.GetHeadPosition();
		while (pos){
			prev_pos=pos;
			cur_socket=m_ControlQueue_list.GetNext(pos);
			if (!prioritySend || !thePrefs.IsIncludeOverhead() || cur_socket->IsPrioritySend()){
				packetSize = cur_socket->GetControlPacketFragSize(minFragSize);
				while (toSend > (int)packetSize || cur_socket->IsPrioritySend() || !thePrefs.IsIncludeOverhead()){
					if (cur_socket->CanBeRemovedFromControlQueue()){
						cur_socket->onUpControlQueue = false;
						m_ControlQueue_list.RemoveAt(prev_pos);
						break;
					}
					else if (cur_socket->IsConnected() && cur_socket->isReadyToSendControlPacket()){
						sent = cur_socket->Send(packetSize);
						if (!sent.success)
							break;
						packetSize = cur_socket->GetControlPacketFragSize(minFragSize);
					}
					else
						break;
				}
			}
		}

		// keep minimum data upload

		if (!uploadinglist.GetCount()) {
			m_SentBytesSinceLastCall += datarateX;
			bandwidthUsed = false;
			sendLocker.Unlock();
			continue;
		}

		if (!prioritySocket && cumulate && !prioritylist.IsEmpty()){
			prioritySocket = prioritylist.RemoveHead();
			if (!prioritySocket->onReserved){
				pos = uploadinglist.Find(prioritySocket);
				ASSERT(pos);
				uploadinglist.RemoveAt(pos);
				if (feedingPos == pos)
					feedingPos = NULL;
				uploadinglist.AddHead(prioritySocket);
			}
			else{
				pos = reserveduploadinglist.Find(prioritySocket);
				ASSERT(pos);
				prioritySocket->onReserved = false;
				reserveduploadinglist.RemoveAt(pos);
				if (reservedFeedingPos == pos)
					reservedFeedingPos = NULL;
				uploadinglist.AddHead(prioritySocket);
			}
			prioritySocket->isForPriorityClient = true;
			pos = NULL;
		}
		//End check
		if (nextMinPacketSize == UINT_MAX || toSend > (int) nextMinPacketSize){
			nextMinPacketSize = UINT_MAX;
			if (cumulate){

				bandwidthUsed = (prioritySend) ? true : false;

				pos=uploadinglist.GetHeadPosition();
				while (pos){
					prev_pos=pos;
					cur_clientSocket = uploadinglist.GetNext(pos);
					if (!bandwidthUsed && cur_clientSocket && cur_clientSocket->isReadyToSendFileData()){
						packetSize=cur_clientSocket->GetDataPacketFragSize(minFragSize);
						while (toSend > (int)packetSize && cur_clientSocket->isReadyToSendFileData() && !prioritySend){
							sent = cur_clientSocket->Send(packetSize);
							datarateX += sent.sentBytesStandardPackets;
							SentToNormal += sent.sentBytesStandardPackets;
							cur_clientSocket->processed=1;
							packetSize=cur_clientSocket->GetDataPacketFragSize(minFragSize);
							if (!sent.success)
								break;
							else {
								cur_clientSocket->m_uReadyTicks++;
								cur_clientSocket->CalcRatio(false);
								cur_clientSocket->SetTimeForSend(thisLoopTick, false);
							}
						}
						if (packetSize > 0 && packetSize < nextMinPacketSize)
							nextMinPacketSize = packetSize;
						if (toSend > (int) packetSize && !prioritySend)
							bandwidthUsed=false;
						else
							bandwidthUsed=true;
					}
					if (!cur_clientSocket->processed)
						cur_clientSocket->processed = 1;
				}

				pos=reserveduploadinglist.GetHeadPosition();
				while (pos){
					cur_clientSocket = reserveduploadinglist.GetNext(pos);
					packetSize=cur_clientSocket->GetDataPacketFragSize(minFragSize);
					if (!bandwidthUsed && cur_clientSocket && cur_clientSocket->isReadyToSendFileData()) {
						while (!prioritySend && toSend > (int)packetSize && cur_clientSocket->isReadyToSendFileData()){
							sent = cur_clientSocket->Send(packetSize);
							datarateX += sent.sentBytesStandardPackets;
							SentToReserved += sent.sentBytesStandardPackets;
							cur_clientSocket->processed = 1;
							packetSize=cur_clientSocket->GetDataPacketFragSize(minFragSize);
							if (!sent.success)
								break;
							else {
								cur_clientSocket->m_uReadyTicks++;
								cur_clientSocket->CalcRatio(false);
								cur_clientSocket->SetTimeForSend(thisLoopTick, false);
							}
						}
						if (packetSize > 0 && packetSize < nextMinPacketSize)
							nextMinPacketSize = packetSize;
						if (toSend > (int) packetSize && !prioritySend)
							bandwidthUsed=false;
						else
							bandwidthUsed=true;
					}
					if (!cur_clientSocket->processed)
						cur_clientSocket->processed = 1;
				}
			}
			else{
				pos=feedingPos;
				uint32 notFullFeeded=uploadinglist.GetCount();
				if (prioritySend)
					bandwidthUsed=true;
				else
					bandwidthUsed=false;
				while (notFullFeeded){
					if (!pos) pos=uploadinglist.GetHeadPosition();
					cur_clientSocket = uploadinglist.GetNext(pos);
					if (cur_clientSocket)
						packetSize = cur_clientSocket->GetDataPacketFragSize(minFragSize);
					if (toSend > (int) packetSize && cur_clientSocket && cur_clientSocket->isReadyToSendFileData() && !prioritySend){
						sent = cur_clientSocket->Send(packetSize);
						datarateX += sent.sentBytesStandardPackets;
						SentToNormal += sent.sentBytesStandardPackets;
						if (!sent.success){
							cur_clientSocket->processed=2;
							notFullFeeded--;
						}
						else {
							cur_clientSocket->m_uReadyTicks++;
							cur_clientSocket->processed = 1;
							cur_clientSocket->CalcRatio(false);
							cur_clientSocket->SetTimeForSend(thisLoopTick, false);
						}
					}
					else if (!cur_clientSocket->processed){
						cur_clientSocket->processed = 2;
						notFullFeeded--;
					}
					else if (cur_clientSocket->processed == 1){
						cur_clientSocket->processed=2;
						notFullFeeded--;
					}
					if (toSend < (int) packetSize)
						bandwidthUsed=true;
					if (cur_clientSocket)
						packetSize = cur_clientSocket->GetDataPacketFragSize(minFragSize);
					if (packetSize > 0 && packetSize < nextMinPacketSize)
						nextMinPacketSize = packetSize;
				}
				feedingPos=pos;

				pos=reservedFeedingPos;
				notFullFeeded=reserveduploadinglist.GetCount();
				while (notFullFeeded){
					if (!pos) pos=reserveduploadinglist.GetHeadPosition();
					cur_clientSocket = reserveduploadinglist.GetNext(pos);
					if (cur_clientSocket)
						packetSize=cur_clientSocket->GetDataPacketFragSize(minFragSize);
					if (!bandwidthUsed  && cur_clientSocket && !prioritySend && toSend > (int) packetSize && cur_clientSocket->isReadyToSendFileData()){
						sent = cur_clientSocket->Send(packetSize);
						datarateX += sent.sentBytesStandardPackets;
						SentToReserved += sent.sentBytesStandardPackets;
						if (!sent.success){
							cur_clientSocket->processed=2;
							notFullFeeded--;
						}
						else {
							cur_clientSocket->m_uReadyTicks++;
							cur_clientSocket->processed = 1;
							cur_clientSocket->CalcRatio(false);
							cur_clientSocket->SetTimeForSend(thisLoopTick, false);
						}
					}
					else if (!cur_clientSocket->processed){
						cur_clientSocket->processed = 2;
						notFullFeeded--;
					}
					else if (cur_clientSocket->processed == 1){
						cur_clientSocket->processed = 2;
						notFullFeeded--;
					}
					if (toSend < (int) packetSize)
						bandwidthUsed=true;
					if (cur_clientSocket)
						packetSize = cur_clientSocket->GetDataPacketFragSize(minFragSize);
					if (packetSize > 0 && packetSize < nextMinPacketSize)
						nextMinPacketSize = packetSize;
				}
				reservedFeedingPos=pos;
			}
		}
		sendLocker.Unlock();

		dataLocker.Lock();
		TransferredData transferitem = {SentToNormal, ::GetTickCount()};
		sentToNormalList.AddTail(transferitem);
		DataSentToNormal+=SentToNormal;

		transferitem.datalen=SentToReserved;
		transferitem.timestamp=::GetTickCount();
		sentToReservedList.AddTail(transferitem);
		DataSentToReserved+=SentToReserved;

		SentToNormal=0;
		SentToReserved=0;

		m_SentBytesSinceLastCall += datarateX;
		dataLocker.Unlock();
	}

	threadEndedEvent->SetEvent();

	sendLocker.Lock();

	m_ControlQueue_list.RemoveAll();
	uploadinglist.RemoveAll();
	reserveduploadinglist.RemoveAll();
	blockedSockets.RemoveAll();
	prioritylist.RemoveAll();
	sendLocker.Unlock();

	return 0;
}
void UploadBandwidthThrottler::ReSortUploadSlots(CUpDownClient* client, bool toReserve){
	if (client){
		sendLocker.Lock();
		// Remove the found Client from UploadBandwidthThrottler
		theApp.uploadBandwidthThrottler->RemoveFromStandardListNoLock(client->socket);
		theApp.uploadBandwidthThrottler->RemoveFromStandardListNoLock((CClientReqSocket*) client->m_pPCUpSocket);
		theApp.uploadBandwidthThrottler->RemoveFromStandardListNoLock((CClientReqSocket*) client->m_pWCUpSocket);
		theApp.uploadBandwidthThrottler->AddToStandardListNoLock(client->GetFileUploadSocket(), toReserve);
		sendLocker.Unlock();
	}
}
// <--- Pawcio: BC - changed
